// WhaleTeqSECG_SDK.h

#if defined(WIN32)
#ifdef WHALETEQ_API

#else
#define WHALETEQ_API extern "C" __declspec(dllimport)
#endif
#elif defined(LINUX)
#define WHALETEQ_API extern "C" __attribute__ ((visibility ("default")))
#endif



//Return value:
//	true:  found plugged SECG Device, build the connection and succeed the initialization
//	false: cant not build any SECG Device connection
WHALETEQ_API bool InitSECG();

//Specify the COM Port number for InitSECG() to connect SECG Device.
//The API should be called before InitSECG().
//Under multiple SECG devices environment, the API can control which SECG to be used by SDK.
WHALETEQ_API void SpecifyCOMPort(int portNum);

WHALETEQ_API void ResetSECG(); //Reset all setting value as default, lead RA ON
WHALETEQ_API void ResetSECG_AllLeadOff(); //Reset all setting value as default, all leads Off

WHALETEQ_API void CloseSECG(); //Close the connection

WHALETEQ_API bool IsConnected();

WHALETEQ_API char* GetUSBPort();

WHALETEQ_API char* GetSerialNo();

WHALETEQ_API char* GetHWInfo(); //Get connected SECG hardware information
WHALETEQ_API char* GetFWVersion(); //Get connected SECG firmware version
WHALETEQ_API char* GetHWVersion(); //Get connected SECG hardware version

WHALETEQ_API char* GetSDKVersion(); //Get SECG SDK version

//Parameter - connected
//	true: SECG Device connected
//  flase: Lost SECG Device connection 
typedef void(__stdcall *DeviceConnectedCB)(bool connected);

//The "DeviceConnectedCB connCB" would be triggered when:
//   New SECG Device connection built when InitSECG() be called, parameter "bool connected" as true.
//   SDK lost connection with SECG Device, parameter "bool connected" as false.
WHALETEQ_API void RegistConnectCB(DeviceConnectedCB connCB);

//Parameter - time
//	unit: second
//	time from the output signal changed(function, amplitude, frequency ...)
//Parameter - amplitude
//	unit: millivolt
typedef void (__stdcall *TimeAmplitudeCB)(double time, double amplitude);

//The "TimeAmplitudeCB cb" would be triggered during SECG outputting signal.
//About 0.006 second(6ms) interval between each callback be triggered
WHALETEQ_API void RegisterTimeAmpCB(TimeAmplitudeCB cb);

//The "TimeAmplitudeCB cb" would be triggered during SECG outputting waveform with pacing signal
WHALETEQ_API void RegisterPacingTimeAmpCB(TimeAmplitudeCB cb);

enum OutputFunction_E {
	Output_Off	= 0,
	Output_Sine,
	Output_Triangle,
	Output_Square,
	Output_RectanglePulse,
	Output_TrianglePulse,
	Output_Exponential,
	Output_ECG2_27,
	Output_IEC227W,
	Output_IEC251W,
	Output_JJG1041,
	Output_JJG1041_HR,
	Output_JJG_Hysteresis,
	Output_ECG_File,
	Output_InputDynamicRange,
	Output_ANE20000
};

enum SpecialWave227_E{
	IEC227W_3A_A1 = 0,
	IEC227W_3B_A2,
	IEC227W_3C_A3,
	IEC227W_3D_A4,
	IEC227W_4A_B1,
	IEC227W_4A_B1x2,
	IEC227W_4A_B1d2,
	IEC227W_4B_B2,
	IEC227W_4B_B2x2,
	IEC227W_4B_B2d2,

};
enum SpecialWave251_E{
	IEC251W_1 = 0,	//CAL05000
	IEC251W_2,		//CAL10000
	IEC251W_3,		//CAL15000
	IEC251W_4,		//CAL20000
	IEC251W_6,		//CAL30000
	IEC251W_10,		//CAL50000
	CAL20002,		//--------- New Added ------------------------
	CAL20100,
	CAL20110,
	CAL20160,
	CAL20200,
	CAL20210,
	CAL20260,
	CAL20500,
	CAL20502,
	CAL40000
};

//Return value:
// 0:  succeed
// -1: Mainfunction not set to IEC227W
// Call after set main func to Output_IEC227w
WHALETEQ_API int SetSpecialWave227(SpecialWave227_E spec);
WHALETEQ_API SpecialWave227_E GetSpecialWave227();

//Return value:
// 0:  succeed
// -1: Mainfunction not set to IEC251W
// Call after set main func to Output_IEC251w
WHALETEQ_API int SetSpecialWave251(SpecialWave251_E spec);
WHALETEQ_API SpecialWave251_E GetSpecialWave251();

//Return value:
// 0:  succeed
// -1: lead (0~2: LeadI/II/III; 3~8: Lead V1~V6)
// Main func would change to Output_ANE20000 after Call 
WHALETEQ_API int SetANE20000Lead(int lead);
WHALETEQ_API int GetANE20000Lead();

//Return value:
//	0:	succeed
//	-1:	SECG not be initialized
WHALETEQ_API int SetOutputFunc(OutputFunction_E func);
WHALETEQ_API OutputFunction_E GetOutputFunc();

//Parameter - amp:
//	value range: -10 ~ 10
//	unit: millivolt(mV)
//Return value:
//	0: succeed
//	-1: SECG not be initialized
//	-2: value out of range
//  -3: "Baseline Reset Test" is working
//Default value after InitSECG(): 1mV
WHALETEQ_API int SetAmplitude(double amp);
WHALETEQ_API double GetAmplitude();
#define DefaultAmplitude	1.0
#define MaxAmplitude		10.0
#define MinAmplitude		-10.0

//Parameter - freq:
//	value range: 0.05 ~ 5, if Output is Pulse Waveforms
//               0.05 ~ 3, if Output is Exponential
//               0.05 ~ 6, if Output is ECG2_27
//               0.05 ~ 500, others
//	unit: hertz
//Return value:
//	0: succeed
//	-1: SECG not be initialized
//	-2: value out of range
//	-3: "Frequency Scan" is working
//  -4: "Baseline Reset Test" is working
//Default value after InitSECG(): 1Hz
WHALETEQ_API int SetFrequency(double freq);
WHALETEQ_API double GetFrequency();
#define DefaultFrequency		1.0
#define	MaxFrequency			500.0
#define MinFrequency			0.05
#define	MaxFrequencyPulseWave	5.0

//Parameter - setting:
//	value range: -300 ~ 300 - DCOffset Variable mode Off
//               -1000 ~ 1000 - DCOffset Variable mode On
//	unit: millivolt(mV)
//Return value:
//	0: succeed
//	-1: SECG not be initialized
//	-2: value out of range
//Default value after InitSECG(): 0mV
WHALETEQ_API int SetDCOffset(int setting);
WHALETEQ_API int GetDCOffset();
#define DefaultDCOffset		0
#define MaxDCOffset			300
#define MinDCOffset			-300
#define MaxDCOffsetVarMode	1000
#define MinDCOffsetVarMode	-1000

//Return value:
//	0: succeed
//	-1: SECG not be initialized
//Default value after InitSECG(): false(Off)
WHALETEQ_API int SetDCOffsetVariableMode(bool OnOff);
WHALETEQ_API bool GetDCOffsetVariableMode();

//Return value:
//	0: succeed
//	-1: SECG not be initialized
//Default value after InitSECG(): false(Off)
WHALETEQ_API int SetDCOffsetCommonMode(bool OnOff);
WHALETEQ_API int SetDCOffsetCommoneMode(bool OnOff); //For backward compatical the typo
WHALETEQ_API bool GetDCOffsetCommonMode();
WHALETEQ_API bool GetDCOffsetCommoneMode(); //For backward compatical the typo

enum OutputLead_E {
	Lead_RA		= 0,	//(R)
	Lead_LA,			//(L)
	Lead_LL,			//(F)
	Lead_V1,
	Lead_V2,
	Lead_V3,
	Lead_V4,
	Lead_V5,
	Lead_V6
};
//Return value:
//	0: succeed
//	-1: SECG not be initialized
//Default value after InitSECG(): Lead_RA - On; all others - Off;
WHALETEQ_API int SetOutputLead(OutputLead_E lead, bool OnOff);
WHALETEQ_API bool GetOutputLead(OutputLead_E lead);

//Parameter - width:
//	value range: 2 ~ 300
//	unit: milisecond(ms)
//Return value:
//	0: succeed
//	-1: SECG not be initialized
//	-2: value out of range
//Default value after InitSECG(): 100ms
WHALETEQ_API int SetPulseWidth(double width);
WHALETEQ_API double GetPulseWidth();
#define DefaultPulseWidth	100
#define MaxPulseWidth		300
#define MinPulseWidth		2

//Return value:
//	0: succeed
//	-1: SECG not be initialized
//Default value after InitSECG(): true(On - 620k&/4.7nF shorted)
WHALETEQ_API int SetInputImpedanceMode(bool OnOff);
WHALETEQ_API bool GetInputImpedanceMode();

enum PacingPulse_E {
	SinglePulse	= 0,
	DoublePulse_150ms,
	DoublePulse_250ms
};
//Return value:
//	0:	succeed
//	-1:	SECG not be initialized
//Default value after InitSECG(): SinglePulse
WHALETEQ_API int SetPacingPulseMode(PacingPulse_E mode);
WHALETEQ_API PacingPulse_E GetPacingPulseMode();

//Parameter - time:
//	value range: 5 ~ 200
//	unit: milisecond(ms)
//Return value:
//	0: succeed
//	-1: SECG not be initialized
//	-2: value out of range
//Default value after InitSECG(): 100ms
WHALETEQ_API int SetQRSDuration(double time);
WHALETEQ_API double GetQRSDuration();
#define DefaultQRSDuration	100
#define MaxQRSDuration	200
#define MinQRSDuration	5

//Parameter - amp:
//	value range: 0 ~ 2.5
//	unit: millivolt(mV)
//Return value:
//	0: succeed
//	-1: SECG not be initialized
//	-2: value out of range
//Default value after InitSECG(): 0.2mV
WHALETEQ_API int SetTWave(double amp);
WHALETEQ_API double GetTWave();
#define DefaultTWave	0.2
#define MaxTWave		2.5
#define MinTWave		0

//Parameter - amp:
//	value range: -700 ~ 700
//	unit: millivolt(mV)
//Return value:
//	0: succeed
//	-1: SECG not be initialized
//	-2: value out of range
//Default value after InitSECG(): 0mV
WHALETEQ_API int SetPacingAmplitude(double amp);
WHALETEQ_API double GetPacingAmplitude();
#define DefaultPacingAmplitude	0
#define MaxPacingAmplitude		700
#define MinPacingAmplitude		-700

//Parameter - time:
//	value range: 0.1 ~ 2
//	unit: milisecond(ms)
//Return value:
//	0: succeed
//	-1: SECG not be initialized
//	-2: value out of range
//Default value after InitSECG(): 2ms
WHALETEQ_API int SetPacingDuration(double time);
WHALETEQ_API double GetPacingDuration();
#define DefaultPacingDuration	2.0
#define MaxPacingDuration		3.0
#define MinPacingDuration		0.1

//Parameter - BPM:
//	value range: 10 ~ 300
//	unit: beats per minute(BPM)
//Return value:
//	0: succeed
//	-1: SECG not be initialized
//	-2: value out of range
//Default value after InitSECG(): 60BPM
WHALETEQ_API int SetPacingRate(unsigned BPM);
WHALETEQ_API unsigned GetPacingRate();
#define DefaultPacingRate	60
#define MaxPacingRate		300
#define MinPacingRate		10

//Parameter - time:
//	value range: 0 ~ 100
//	unit: milisecond(ms)
//Return value:
//	0: succeed
//	-1: SECG not be initialized
//	-2: value out of range
//Default value after InitSECG(): 0ms
WHALETEQ_API int SetOvershootTimeConstant(unsigned time);
WHALETEQ_API unsigned GetOvershootTimeConstant();
#define DefaultOvershootTimeConstant	0
#define MaxOvershootTimeConstant		100
#define MinOvershootTimeConstant		0

//Parameter - amp:
//	value range: -2 ~ 2
//	unit: millivolt(mV)
//Return value:
//	0: succeed
//	-1: SECG not be initialized
//	-2: value out of range
//Default value after InitSECG(): 0mV -> Auto set to -0.25 * Pacing Amplitude
WHALETEQ_API int SetPacingOvershootAmp(double amp);
WHALETEQ_API double GetPacingOvershootAmp();
#define DefaultPacingOvershootAmp	0   //Auto set to -0.25 * Pacing Amplitude
#define MaxPacingOvershootAmp		2
#define MinPacingOvershootAmp		-2

//Parameter - freq
//	unit: hertz
//	frequency value changed during "Baseline Reset Test" or "Frequency Scanning"
typedef void (__stdcall *SyncFreqCB)(double freq);

WHALETEQ_API void RegisterSyncFreqCB(SyncFreqCB cb);

enum BaselineReset_E {
	Baseline_Off	= 0,
	Baseline_50Hz,
	Baseline_60Hz,
	Baseline_80Hz,
	Baseline_100Hz
};
//Return value:
//	0: succeed
//	-1: SECG not be initialized
//	-2: current main output function not be "Output_Sine"
//Default value after InitSECG(): Baseline_Off
//"Output Function" must be set to "Output_Sine" before setting as "non - Baseline_Off" value
//After setting as "non - Baseline_Off" value, "Amplitude" would be locked to "2.5mV"
//and "Frequency" would be locked as "50/60/80/100" Hz.
WHALETEQ_API int SetBaselineResetTest(BaselineReset_E mode);
WHALETEQ_API BaselineReset_E GetBaselineResetTest();

enum ECG2_27_NoiseFreq_E {
	Noise_None	= 0,
	Noise_50Hz,
	Noise_60Hz
};
//Parameter - freq:
//	Default value after InitSECG(): Noise_None
//Parameter - amp:
//	value range: 0.01 ~ 10
//	unit: millivolt(mV)
//	Default value after InitSECG(): 0.1mV
//Return value:
//	0: succeed
//	-1: SECG not be initialized
//	-2: current main output function not be "Output_ECG2_27"
//	-3: amp out of range 
WHALETEQ_API int SetECG2_27_Noise(ECG2_27_NoiseFreq_E freq, double amp);
WHALETEQ_API ECG2_27_NoiseFreq_E GetECG2_27_NoiseFreq();
WHALETEQ_API double GetECG2_27_NoiseAmp();
#define DefaultECG2_27_NoiseAmp	0.1
#define MaxECG2_27_NoiseAmp		10
#define MinECG2_27_NoiseAmp		0.01


//Parameter - enableDriftTest
//    true, enable AAMI EC 13 Drift test; adds 4mVpp 0.1Hz triangle waveform to the ECG signal 
//    false, disable AAMI EC 13 Drift test
//Return value:
//	0: succeed
//	-1: SECG not be initialized
WHALETEQ_API int SetECG2_27_AAMIDriftTest (bool enableDriftTest);
WHALETEQ_API bool GetECG2_27_AAMIDriftTest ();

//Parameter - amp:
//	value range: 0 ~ 5
//	unit: millivolt(mV)
//	Default value after InitSECG(): 1mV
//Parameter - freq:
//	value range: 10 ~ 50
//	unit: hertz
//	Default value after InitSECG(): 40Hz
//Return value:
//	0: succeed
//	-1: SECG not be initialized
//	-2: current main output function not be "Output_Square"
//	-3: amp out of range
//	-4: freq out of range
WHALETEQ_API int SetDynamicRangeTest(bool OnOff, double amp, double freq);
WHALETEQ_API bool GetDynamicRangeTest();
WHALETEQ_API double GetDynamicRangeTestAmp();
WHALETEQ_API double GetDynamicRangeTestFreq();
#define DefaultDynamicRangeTestAmp	1.0
#define MaxDynamicRangeTestAmp		5.0
#define MinDynamicRangeTestAmp		0
#define DefaultDynamicRangeTestFreq	40
#define MaxDynamicRangeTestFreq		50
#define MinDynamicRangeTestFreq		10

//Parameter - func
//	main function changed during "Frequency Scanning"
typedef void (__stdcall *SyncMainFuncCB)(OutputFunction_E func);

WHALETEQ_API void RegisterSyncMainFuncCB(SyncMainFuncCB cb);

//Parameter - startFreq:
//	value range: 0.67 ~ 500
//	unit: hertz
//	Default value after InitSECG(): 0.67Hz
//Parameter - stopFreq:
//	value range: 0.01 ~ 500
//	unit: hertz
//	Default value after InitSECG(): 150Hz
//Parameter - duration:
//	value range: 10 ~ 180
//	unit: millisecond(ms)
//	Default value after InitSECG(): 30ms
//Return value:
//	0: succeed
//	-1: SECG not be initialized
//	-2: startFreq out of range
//	-3: stopFreq out of range
//	-4: duration out of range
//"Output Function" would be set to "Output_Sine" when setting "OnOff" as true,
//then "Frequency" would be locked(SetFrequency() return -3) for periodly scanning change.
//"Output Function" would be set to "Output_Off" when setting "OnOff" as false.
WHALETEQ_API int SetFreqScanSine(bool OnOff, double startFreq, 
											double stopFreq, int duration);
WHALETEQ_API bool GetFreqScanSine();
WHALETEQ_API double GetFreqScanSineStartFreq();
WHALETEQ_API double GetFreqScanSineStopFreq();
WHALETEQ_API int GetFreqScanSineDuration();
#define DefaultScanSineStartFreq	0.67
#define DefaultScanSineStopFreq		150
#define MaxScanSineFreq				500.0
#define MinScanSineFreq				0.67
#define MinScanSineStopFreq			0.01
#define DefaultScanSineDuration		30
#define MaxScanSineDuration			180
#define MinScanSineDuration			10

//Start Frequency as 3BPM(0.05Hz)
//Stop Frequency as 30BPM(0.5Hz)
//Duration as 30ms
//"Output Function" would be set to "Output_ECG2_27" when setting "OnOff" as true,
//then "Frequency" would be locked(SetFrequency() return -3) for periodly scanning change.
//"Output Function" would be set to "Output_Off" when setting "OnOff" as false.
WHALETEQ_API int SetFreqScanECG(bool OnOff);
WHALETEQ_API bool GetFreqScanECG();

//Return value:
// >0: succeed, the number of sample data loaded
// -1: Open file failed
// -2: Can't solve sample rate in Line 1
// -3: Can't solve sample number in Line 2
// -4: Raw data file is too large to fit the memory
WHALETEQ_API int LoadECGtxt(wchar_t* file);
WHALETEQ_API int LoadECGbin(wchar_t* file);

//3rd Party License Integration ----------------------------
WHALETEQ_API bool CheckoutLicenseFeature1();
WHALETEQ_API bool CheckoutLicenseFeature2();
WHALETEQ_API bool CheckoutLicenseFeature3();
WHALETEQ_API char* GetHardwareID(); //To Generate License Key
WHALETEQ_API bool ActiveLicenseKey(char* Key);

WHALETEQ_API unsigned char* GetLicenseKey();
WHALETEQ_API void WriteLicenseKey(unsigned char** B, int len, int offset);

WHALETEQ_API int GetAD();

WHALETEQ_API bool TriggerDeviceChanged();
