from ctypes import *
from secg40 import *
import sys
import time

device = SECG40()


def TestMisc():
    print('TestMisc -----------------------------------------------------')
    print('FirmWare Version is: ', device.GetFWVersion())
    print('SDK Version is: ', device.GetSDKVersion())
    print('Device Serial Number is: ', device.GetHWInfo())


def TestDCOffset():
    print('TestDCOffset -----------------------------------------------------')
    # DC Offset Related APIs
    DCOffset = device.GetDCOffset()
    print('Initial DC Offset: ', DCOffset, ' mV')

    DCVarMode = device.GetDCOffsetVariableMode()
    if 0 != DCVarMode:
        print('Initial DC Offset Variable Mode: On')
    else:
        print('Initial DC Offset Variable Mode: Off')

    DCComMode = device.GetDCOffsetCommoneMode()
    if 0 != DCComMode:
        print('Initial DC Offset Common Mode: On')
    else:
        print('Initial DC Offset Common Mode: Off')

    # Under "Non-Variable Mode", only "300mV" and "-300mV" 2 value can be set
    DCOffset = 300
    device.SetDCOffset(DCOffset)
    device.SetOutputFunc(OutputFunction_E.Output_Sine.value)
    print('Output Sine signal with 300mV DC offset')
    time.sleep(5)

    DCOffset = -300
    device.SetDCOffset(DCOffset)
    print('Output Sine signal with -300mV DC offset')
    time.sleep(5)

    # Under "Variable Mode", between "-1000mV"  and "1000mV" integer value van be set
    device.SetDCOffsetVariableMode(1)
    DCVarMode = device.GetDCOffsetVariableMode()
    if 0 != DCVarMode:
        print('Turn On DC Offset Variable Mode.')

    DCOffset = 150
    device.SetDCOffset(DCOffset)  # Output Sine signal with 150mV DC offset
    print('Output Sine signal with 150mV DC offset')
    time.sleep(5)

    DCOffset = -150
    device.SetDCOffset(DCOffset)  # Output Sine signal with -150mV DC offset
    print('Output Sine signal with -150mV DC offset')
    time.sleep(5)

    device.SetOutputFunc(OutputFunction_E.Output_Off.value)
    device.SetDCOffset(0)
    # Common Mode only work without Variable Mode
    device.SetDCOffsetVariableMode(0)

    device.SetDCOffsetCommoneMode(1)
    DCComMode = device.GetDCOffsetCommoneMode()
    if 0 != DCComMode:
        print('Turn On DC Offset Common Mode.')

    DCOffset = 300
    device.SetDCOffset(DCOffset)
    device.SetOutputFunc(OutputFunction_E.Output_Sine.value)
    print('Output Sine signal with 300mV DC offset in common mode')
    time.sleep(5)


def TestImpedance():
    print('TestImpedance -----------------------------------------------------')

    ImpedMode = device.GetInputImpedanceMode()
    if 0 != ImpedMode:
        print('Initial Input Impedance Mode: On (without 620K Ohm impedance)')
    else:
        print('Initial Input Impedance Mode: Off (with 620K Ohm impedance)')

    amp = c_double(5)
    device.SetAmplitude(amp)

    freq = c_double(10)
    device.SetFrequency(freq)

    device.SetOutputFunc(OutputFunction_E.Output_Sine.value)
    print('Output 5mV 10Hz Sine without 620K Ohm impedance signal to measure Reference Vpp')
    time.sleep(5)

    device.SetOutputFunc(OutputFunction_E.Output_Off.value)
    device.SetInputImpedanceMode(0)
    ImpedMode = device.GetInputImpedanceMode()
    if 0 == ImpedMode:
        print('Turn off Input Impedance Mode: with 620K Ohm impedance')
    device.SetOutputFunc(OutputFunction_E.Output_Sine.value)
    print('Output 5mV 10Hz Sine with 620K Ohm impedance signal to measure Comparison Vpp')
    time.sleep(5)


def TestPulseWidth():
    print('TestPulseWidth -----------------------------------------------------')

    pulseWidth = device.GetPulseWidth()
    print('Initial Pulse Widht: ', pulseWidth, ' ms')

    device.SetOutputFunc(OutputFunction_E.Output_RectanglePulse.value)
    print('Output ', OutputFunction_E(device.GetOutputFunc()),
          ' signal with ', pulseWidth, ' ms Pulse Width')
    time.sleep(5)

    device.SetOutputFunc(OutputFunction_E.Output_Off.value)
    device.SetOutputFunc(OutputFunction_E.Output_TrianglePulse.value)
    print('Output ', OutputFunction_E(device.GetOutputFunc()),
          ' signal with ', pulseWidth, ' ms Pulse Width')
    time.sleep(5)

    device.SetOutputFunc(OutputFunction_E.Output_Off.value)
    pulseWidth = c_double(200)
    if 0 == device.SetPulseWidth(pulseWidth):
        print('Set Pulse Width to ', device.GetPulseWidth())

    device.SetOutputFunc(OutputFunction_E.Output_RectanglePulse.value)
    print('Output ', OutputFunction_E(device.GetOutputFunc()),
          ' signal with ', device.GetPulseWidth(), ' ms Pulse Width')
    time.sleep(5)


if 0 == device.InitSECG():
    print('InstSECG() failed')
else:
    print('InitSECG() succeed')

    TestMisc()

    TestDCOffset()
    device.ResetSECG()

    TestImpedance()
    device.ResetSECG()

    TestPulseWidth()
